package com.loit.loitcasclient.controller;

import com.loit.loitcasclient.config.ClientAppConfig;
import com.loit.loitcasclient.service.ClientLoginService;
import com.loit.loitcasclient.utils.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.client.RestTemplate;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

@Controller
@RequestMapping(value = "cas")
public class ClientLoginController {

    @Autowired
    private RestTemplate restTemplate;
    @Autowired(required = false)
    private ClientAppConfig clientAppConfig;
    @Autowired
    private ClientUserUtil userUtil;
    @Autowired
    private ClientLoginService clientLoginService;
	protected Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private LoitLog log;

    /**
     * 免登录入口
     * 页面自动填充参数模拟点击登录按钮
     * 登录服务器需要配置门户的login/free验证接口
     */
    @RequestMapping(value = "login/free", method = RequestMethod.GET)
    public void loginFree(
            @RequestParam(value="username",required = true) String username,
            @RequestParam(value="password",required = true) String password,
            HttpServletResponse response) {
        clientLoginService.loginFree(username, password, response);
    }

    /**
     * 登录入口
     * 采用预登录，预登录链接可被收藏
     */
    @RequestMapping(value = "login/index", method = RequestMethod.GET)
    public void loginFree(
            HttpServletResponse response) {
        clientLoginService.loginFree(null, null, response);
    }

    /**
     * 登录入口
     * 登录页面需要手动输入用户名密码验证码
     */
    @RequestMapping(value = "login", method = RequestMethod.GET)
    public void login(
            @RequestParam(value="url",required = false) String url,
            HttpServletResponse response,HttpServletRequest request) {
        clientLoginService.login(url, response);
    }

    /**
     * 获取令牌
     */
    @RequestMapping(value = "accessToken", method = RequestMethod.GET)
    public void accessToken(
            @RequestParam("code") String code,
            HttpServletResponse response,
            HttpServletRequest request) throws IOException {

    	String accessTokenUrl = clientAppConfig.accessTokenUrl().replace("{code}", code);
    	log.info("ClientLoginController accessToken() accessTokenUrl:" + accessTokenUrl);
        String accessToken = ClientTokenUtil.accessToken(accessTokenUrl, restTemplate);
        log.info("accessToken:"+accessToken);
        response.addCookie(ClientCookieUtil.instance("accessToken", accessToken, clientAppConfig.getAccessTokenCookieTimOut()));

        String url = ClientBase64Util.decoder(ClientCookieUtil.get(request.getCookies(), "url"));
        log.info("ClientLoginController accessToken() Redirect url 获取令牌后重定向至:"+url);
        response.sendRedirect(url);
    }

    /**
     * 登出
     */
    @ResponseBody
    @RequestMapping(value = "logout", method = RequestMethod.GET)
    public void logout(HttpServletResponse response) throws IOException {
        // cookie 里设置门户首页url
    	log.info("cookie 里设置门户首页url"+clientAppConfig.getPortalIndexUrl());
        response.addCookie(ClientCookieUtil.instance("url", ClientBase64Util.encoder(clientAppConfig.getPortalIndexUrl()), clientAppConfig.getAccessTokenCookieTimOut()));

        // 跳转到登陆接口
        log.info("ClientLoginController logout() Redirect logoutUrl 登出后重定向至:"+clientAppConfig.logoutUrl());
        response.sendRedirect(clientAppConfig.logoutUrl());
//        clientLoginService.logout(response);
    }

//    /**
//     * 登出后跳转到登录页, 登录成功后跳转到门户首页
//     */
//    @RequestMapping(value = "logoutIndex", method = RequestMethod.GET)
//    public void logoutIndex(HttpServletResponse response) throws IOException {
//        // cookie 里设置门户首页url
//        response.addCookie(ClientCookieUtil.instance("url", ClientBase64Util.encoder(clientAppConfig.getPortalIndexUrl()), clientAppConfig.getAccessTokenCookieTimOut()));
//
//        // 跳转到登陆接口
//        String codeUrl = clientAppConfig.getCasServerUrl() + "/cas/login";
//        response.sendRedirect(codeUrl);
//    }

    /**
     * 获取登录用户名-从cookie中获取
     */
    @ResponseBody
    @RequestMapping(value = "userId/cookie", method = RequestMethod.GET)
    public Map userId(HttpServletRequest request) {
        String accessToken = ClientCookieUtil.get(request.getCookies(), "accessToken");
        String userName = userUtil.getUserId(accessToken);
        Map<String, Object> map = new HashMap<>();
        map.put("userName", userName);
        return map;
    }

    /**
     * 获取登录用户名-从登录服务器中获取
     */
    @ResponseBody
    @RequestMapping(value = "userId", method = RequestMethod.GET)
    public Map userId(@RequestParam("accessToken") String accessToken) {
        String userName = userUtil.getUserId(accessToken);
        Map<String, Object> map = new HashMap<>();
        map.put("userName", userName);
        return map;
    }

    /**
     * 测试方法
     */
    @ResponseBody
    @RequestMapping(value = "test", method = RequestMethod.GET)
    public Map<String, Object> cookieAccessToken() {
        Map<String, Object> map = new HashMap<>();
        map.put("value", "Success.");
        return map;
    }
}
