package com.loit.loitcasclient.service;

import java.io.IOException;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.loit.loitcasclient.config.ClientAppConfig;
import com.loit.loitcasclient.utils.ClientBase64Util;
import com.loit.loitcasclient.utils.ClientCookieUtil;
import com.loit.loitcasclient.utils.LoitLog;

@Service
public class ClientLoginService {

    @Autowired
    private LoitLog logger;
    
    @Autowired(required = false)
    private ClientAppConfig clientAppConfig;

    /**
     * 一般登录行为
     */
    public void login(String url, HttpServletResponse response){
        try {
            // 不需要登录的页面, 直接转发
            List<String> forwards = clientAppConfig.getForwards();
            if(forwards!=null && forwards.size()>0 && !StringUtils.isEmpty(url)){
                String _url = ClientBase64Util.decoder(url);
                for (String forward : forwards) {
                    String path = forward.substring(0, forward.indexOf("-"));
                    if(_url.contains(path)){
                        String uri = forward.substring(forward.indexOf("-") + 1);
                        response.sendRedirect(uri);
                        logger.info("ClientLoginService login() Redirect uri:"+uri); 
                        return;
                    }
                }
            }

            // 如果没有url参数, 就使用portal门户的主页地址(表示这是给portal项目配置的)
            if(StringUtils.isEmpty(url)){
                url = clientAppConfig.getPortalIndexUrl();
            }else{
                // Demon modify by 2020-05-13： 临时修改为getWebUrl，以后删除该逻辑
                url = clientAppConfig.getWebUrl() + ClientBase64Util.decoder(url);
            }

            logger.info("前端地址："+url);

            url = ClientBase64Util.encoder(url);
            response.addCookie(ClientCookieUtil.instance("url", url, clientAppConfig.getAccessTokenCookieTimOut()));

            
            String codeUrl = clientAppConfig.codeUrl();
            logger.info("ClientLoginService login() Redirect codeUrl 登录后重定向至:"+codeUrl); 
            response.sendRedirect(codeUrl);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 免登陆行为
     * 预登录一次服务器, 登录成功后走正常流程
     */
    public void loginFree(String username, String password, HttpServletResponse response){
        String loginUrl = clientAppConfig.getCasServerUrl() + "/cas/login?service="+clientAppConfig.getAppServerUrl()+"/cas/login";

        /*
         * 如果username和password参数不为空，则免登录
         * 登录页面js会吧用户名密码填充到文本框，模拟点击登录按钮进行免登录操作。
         * 登录服务器配置登录校验使用门户的login/free接口。
         */
        if(!StringUtils.isEmpty(username) && !StringUtils.isEmpty(password)){
            loginUrl += "&username=" + username + "&password" + password;
        }

        try {
            logger.info("ClientLoginService loginFree() Redirect loginUrl:"+loginUrl); 
            response.sendRedirect(loginUrl);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
