package com.loit.loitcasclient.utils;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;
import java.util.stream.Collectors;

import org.springframework.util.StringUtils;

/**
 * 本地缓存工具类
 */
public class ClientLocalCacheUtil {
    /**
     * 缓存
     */
    private static Map<String, Object> cache = new HashMap<>();

    /**
     * 缓存产生时间
     */
    private static Map<String, Long> cacheTime = new HashMap<>();

    /**
     * 缓存过期时间(秒)
     */
    private static Map<String, Integer> cacheOutTime = new HashMap<>();

    /**
     * 创建定时任务每24小时清理一次缓存
     */
    static{
        Timer timer = new Timer();
        timer.schedule(new TimerTask() {
            @Override
            public void run() {
                refresh();
            }
        },0,60000*60*24);
    }

    /**
     * 缓存刷新,清除过期数据
     */
    public static void refresh(){
        for (String key : cacheOutTime.keySet()) {
            ClientLocalCacheUtil.get(key);
        }
    }

    /**
     * 设置缓存
     */
    public static void set(String key, Object value){
        ClientLocalCacheUtil.set(key, value, null);
    }

    /**
     * 设置有过期时间的缓存
     * @param outTimeSecond 过期时间秒
     */
    public static void set(String key, Object value, Integer outTimeSecond){
        cache.put(key, value);
        cacheTime.put(key, (new Date()).getTime());
        cacheOutTime.put(key, null);
        if(null != outTimeSecond){
            cacheOutTime.put(key, outTimeSecond);
        }
    }

    /**
     * 获取缓存
     */
    public static Object get(String key){
        Integer outTime = cacheOutTime.get(key);
        if(outTime != null) {
	        Long time = cacheTime.get(key);
	        
	        if(null == time) {
	        	return null;
	        }
	        
	        time = cacheTime.get(key)/1000;
	        Long currTime = (new Date()).getTime()/1000;
	
	        // 超期了, 清除缓存
	        if(currTime - time >= outTime){
	            cache.remove(key);
	            cacheTime.remove(key);
	            cacheOutTime.remove(key);
	            return null;
	        }
	    }

        return cache.get(key);
    }

    /**
     * 根据key的前缀获取keys列表
     * @param prefix key前缀
     * @return keys列表
     */
    public static List<String> keys(String prefix){
        Set<String> keys = cache.keySet();
        return keys.stream().filter(e -> e.startsWith(prefix)).distinct().collect(Collectors.toList());
    }

    /**
     * 根据keys列表获取值列表
     * @param keys
     * @return
     */
    public static List<String> values(List<String> keys){
        if(StringUtils.isEmpty(keys)){
            return null;
        }

        List<String> values = new ArrayList<>();
        for (String key : keys) {
            values.add((String)get(key));
        }

        return values;
    }
}
